@@ -3,7 +3,7 @@
  *
  * hf-storage.c : storage device support
  *
- * Copyright (C) 2006 Jean-Yves Lefort <jylefort@FreeBSD.org>
+ * Copyright (C) 2006, 2007 Jean-Yves Lefort <jylefort@FreeBSD.org>
  *                    Joe Marcus Clarke <marcus@FreeBSD.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -102,7 +102,9 @@ hf_storage_get_disk_names (GError **err)
 static gboolean
 hf_storage_class_is_partitionable (const char *geom_class)
 {
-  return (! strcmp(geom_class, "MBR") || ! strcmp(geom_class, "GPT") ||
+  return (! strcmp(geom_class, "MBR") ||
+          ! strcmp(geom_class, "MBREXT") ||
+	  ! strcmp(geom_class, "GPT") ||
           ! strcmp(geom_class, "APPLE") || ! strcmp(geom_class, "SUN"));
 }
 
@@ -143,7 +145,8 @@ hf_storage_geom_is_swap (const Geom_Object *geom_obj)
   g_return_val_if_fail(geom_obj != NULL, FALSE);
 
   return (! strcmp(geom_obj->class, "BSD") && geom_obj->type == FS_SWAP)
-    || (! strcmp(geom_obj->class, "MBR")
+	|| ((! strcmp(geom_obj->class, "MBR") ||
+             ! strcmp(geom_obj->class, "MBREXT"))
 	&& (geom_obj->type == 0x18		/* AST Windows swapfile */
 	    || geom_obj->type == 0x42		/* SFS or Linux swap */
 	    || geom_obj->type == 0x82		/* Linux swap or Solaris x86 */
@@ -641,7 +644,6 @@ void
 hf_storage_device_enable (HalDevice *device)
 {
   g_return_if_fail(HAL_IS_DEVICE(device));
-  g_return_if_fail(devname != NULL);
 
   hal_device_property_set_string(device, "storage.bus", "platform");
   hal_device_property_set_string(device, "storage.drive_type", "disk");
@@ -652,9 +654,9 @@ hf_storage_device_enable (HalDevice *device)
   hal_device_property_set_bool(device, "storage.media_check_enabled", FALSE);
   hal_device_property_set_bool(device, "storage.automount_enabled_hint", TRUE);
   hal_device_property_set_bool(device, "storage.no_partitions_hint", FALSE);
+  hal_device_property_set_bool(device, "storage.removable.support_async_notification", FALSE);
 
   hal_device_property_set_string(device, "storage.originating_device", NULL);
-  hal_device_property_set_string(device, "storage.physical_device", NULL);
   hal_device_property_set_string(device, "storage.model", NULL);
   hal_device_property_set_string(device, "storage.vendor", NULL);
 
@@ -686,6 +688,8 @@ hf_storage_device_enable_cdrom (HalDevice *device)
   hal_device_property_set_bool(device, "storage.no_partitions_hint", TRUE);
   /* the linux backend sets this one */
   hal_device_property_set_bool(device, "storage.requires_eject", TRUE);
+  /* allow the storage addon to watch for media changes */
+  hal_device_property_set_bool(device, "storage.removable.support_async_notification", FALSE);
 
   /* some of these will be set by probe-storage */
   hal_device_property_set_bool(device, "storage.cdrom.cdr", FALSE);
