@@ -36,7 +36,11 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#include <linux/input.h>
+#ifdef HAL_LINUX_INPUT_HEADER_H
+  #include HAL_LINUX_INPUT_HEADER_H
+#else
+  #include <linux/input.h>
+#endif
 #include <glib/gmain.h>
 #include <glib/gprintf.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -171,7 +175,11 @@ static char *key_name[KEY_MAX + 1] = {
 #ifndef KEY_WLAN
 #define KEY_WLAN 238
 #endif
-	[KEY_WLAN] = "wlan"
+	[KEY_WLAN] = "wlan",
+#ifndef KEY_ZOOM
+#define KEY_ZOOM 0x174
+#endif
+	[KEY_ZOOM] = "zoom"
 };
 
 
@@ -206,7 +214,7 @@ event_io (GIOChannel *channel, GIOCondition condition, gpointer data)
 	GError *gerror = NULL;
 	gsize read_bytes;
 
-	if (condition & (G_IO_HUP | G_IO_ERR))
+	if (condition & (G_IO_HUP | G_IO_ERR | G_IO_NVAL))
 		return FALSE;
 
 	/** tbh, we can probably assume every time we read we have a whole
@@ -244,6 +252,11 @@ event_io (GIOChannel *channel, GIOCondition condition, gpointer data)
 			case SW_HEADPHONE_INSERT:
 				name = "headphone_insert";
 				break;
+#ifdef SW_RADIO
+			case SW_RADIO:
+				name = "radio";
+				break;
+#endif
 			}
 			if (name != NULL) {
 				long bitmask[NBITS(SW_MAX)];
@@ -284,7 +297,7 @@ event_io (GIOChannel *channel, GIOCondition condition, gpointer data)
 					}
 				}
 			}
-		} else if (input_data->event.type == EV_KEY && key_name[input_data->event.code] != NULL && input_data->event.value == 1) {
+		} else if (input_data->event.type == EV_KEY && key_name[input_data->event.code] != NULL && input_data->event.value) {
 			dbus_error_init (&error);
 			libhal_device_emit_condition (ctx, input_data->udi,
 						      "ButtonPressed",
@@ -300,19 +313,14 @@ event_io (GIOChannel *channel, GIOCondition condition, gpointer data)
 static void
 destroy_data (InputData *data)
 {
-	HAL_DEBUG (("Removing GIOChannel for'%s'", data->udi));
-
-	/* Null out the GIOChannel in the hash table, but
-	 * leave the key in for DeviceRemoved
-	 */
-	g_hash_table_replace (inputs, g_strdup(data->udi), NULL);
+	HAL_DEBUG (("Input device '%s' destroyed, free data.", data->udi));
 
 	g_free (data);
 }
 
 
 static void
-update_proc_title ()
+update_proc_title (void)
 {
 	GList *lp;
 	gchar *new_command_line, *p;
@@ -353,6 +361,7 @@ add_device (LibHalContext *ctx,
 
 	if ((device_file = libhal_ps_get_string (properties, "input.device")) == NULL) {
 		HAL_ERROR(("%s has no property input.device", udi));
+		g_free (data);
 		return;
 	}
 
@@ -365,6 +374,7 @@ add_device (LibHalContext *ctx,
 	eventfp = open(device_file, O_RDONLY | O_NONBLOCK);
 	if (!eventfp) {
 		HAL_ERROR(("Unable to open %s for reading", device_file));
+		g_free (data);
 		return;
 	}
 
@@ -379,7 +389,7 @@ add_device (LibHalContext *ctx,
 
 	g_hash_table_insert (inputs, g_strdup(udi), channel);
 	g_io_add_watch_full (channel,
-			     G_PRIORITY_DEFAULT, G_IO_IN | G_IO_ERR | G_IO_HUP,
+			     G_PRIORITY_DEFAULT, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 			     event_io, data, (GDestroyNotify) destroy_data);
 }
 
@@ -404,8 +414,10 @@ remove_device (LibHalContext *ctx,
 		return;
 	}
 
-	if (channel)
+	if (channel) {
+		g_io_channel_shutdown(channel, FALSE, NULL);
 		g_io_channel_unref (channel);
+	}
 
 	g_hash_table_remove (inputs, udi);
 
@@ -439,7 +451,7 @@ main (int argc, char **argv)
 
 	hal_set_proc_title_init (argc, argv);
 
-	/* setup_logger (); */
+	setup_logger ();
 
 	dbus_error_init (&error);
 	if ((ctx = libhal_ctx_init_direct (&error)) == NULL) {
