@@ -57,6 +57,34 @@
 
 #include "blockdev.h"
 
+static gchar *
+strdup_valid_utf8 (const char *str)
+{
+        char *endchar;
+        char *newstr;
+        unsigned int fixes;
+
+        if (str == NULL)
+                return NULL;
+
+        newstr = g_strdup (str);
+
+        fixes = 0;
+        while (!g_utf8_validate (newstr, -1, (const char **) &endchar)) {
+                *endchar = '_';
+                ++fixes;
+        }
+
+        /* If we had to fix more than 20% of the characters, give up */
+        if (fixes > 0 && g_utf8_strlen (newstr, -1) / fixes < 5) {
+            g_free (newstr);
+            newstr = g_strdup("");
+        }
+
+        return newstr;
+}
+
+
 /*--------------------------------------------------------------------------------------------------------------*/
 
 static gboolean
@@ -77,49 +105,57 @@ blockdev_compute_udi (HalDevice *d)
 		uuid = hal_device_property_get_string (d, "volume.uuid");
 
 		if (uuid != NULL && strlen (uuid) > 0) {
-			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "/org/freedesktop/Hal/devices/volume_uuid_%s", uuid);
+			hald_compute_udi (udi, sizeof (udi),
+					  "/org/freedesktop/Hal/devices/volume_uuid_%s", uuid);
 		} else if (volumelabel != NULL && strlen (volumelabel) > 0) {
-			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "/org/freedesktop/Hal/devices/volume_label_%s", volumelabel);
+			hald_compute_udi (udi, sizeof (udi),
+					  "/org/freedesktop/Hal/devices/volume_label_%s", volumelabel);
 		} else if (hal_device_property_get_bool(d, "volume.is_disc") &&
 			   hal_device_property_get_bool(d, "volume.disc.is_blank")) {
 			/* this should be a empty CD/DVD */
-			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-                                             "/org/freedesktop/Hal/devices/volume_empty_%s",
-					      hal_device_property_get_string (d, "volume.disc.type"));
+			hald_compute_udi (udi, sizeof (udi),
+                                          "/org/freedesktop/Hal/devices/volume_empty_%s",
+					  hal_device_property_get_string (d, "volume.disc.type"));
 		} else {
 			/* fallback to partition number, size */
-			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "/org/freedesktop/Hal/devices/volume_part%d_size_%lld", 
-					      hal_device_property_get_int (d, "volume.partition.number"),
-					      hal_device_property_get_uint64 (d, "volume.size"));
+			hald_compute_udi (udi, sizeof (udi),
+					  "/org/freedesktop/Hal/devices/volume_part%d_size_%lld",
+					  hal_device_property_get_int (d, "volume.partition.number"),
+					  hal_device_property_get_uint64 (d, "volume.size"));
 		}
 		g_free(volumelabel);
 	} else {
 		const char *model;
 		const char *serial;
+		const char *bus;
+		const char *type;
 
 		model = hal_device_property_get_string (d, "storage.model");
 		serial = hal_device_property_get_string (d, "storage.serial");
+		bus = hal_device_property_get_string (d, "storage.bus");
+		type = hal_device_property_get_string (d, "storage.drive_type");
 
 		if (serial != NULL) {
-			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "/org/freedesktop/Hal/devices/storage_serial_%s", 
-					      serial);
+			hald_compute_udi (udi, sizeof (udi),
+					  "/org/freedesktop/Hal/devices/storage_serial_%s",
+					  serial);
 		} else if ((model != NULL) && (strlen(model) != 0) ) {
-			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "/org/freedesktop/Hal/devices/storage_model_%s", 
-					      model);
+			hald_compute_udi (udi, sizeof (udi),
+					  "/org/freedesktop/Hal/devices/storage_model_%s",
+					  model);
+		} else if ((bus != NULL) && (type != NULL)){
+			hald_compute_udi (udi, sizeof (udi),
+					  "%s_storage_%s_%s",
+					  hal_device_property_get_string (d, "storage.originating_device"),
+					  bus, type);
 		} else {
-			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-					      "%s_storage", 
-					      hal_device_property_get_string (d, "storage.originating_device"));
+			hald_compute_udi (udi, sizeof (udi),
+					  "%s_storage",
+					  hal_device_property_get_string (d, "storage.originating_device"));
 		}
 	}
 
 	hal_device_set_udi (d, udi);
-	hal_device_property_set_string (d, "info.udi", udi);
 
 	return TRUE;
 }
@@ -169,7 +205,6 @@ blockdev_refresh_mount_state (HalDevice *d)
 {
 	FILE *f;
 	struct mntent mnt;
-	struct mntent *mnte;
 	char buf[1024];
 	GSList *volumes = NULL;
 	GSList *volume;
@@ -190,7 +225,7 @@ blockdev_refresh_mount_state (HalDevice *d)
 		goto exit;
 
 	/* loop over /proc/mounts */
-	while ((mnte = getmntent_r (f, &mnt, buf, sizeof(buf))) != NULL) {
+	while (getmntent_r (f, &mnt, buf, sizeof(buf)) != NULL) {
 		struct stat statbuf;
 		dev_t devt;
 
@@ -352,7 +387,7 @@ generate_fakevolume_hotplug_event_add_for_storage_device (HalDevice *d)
 
 	snprintf (fake_sysfs_path, sizeof(fake_sysfs_path), "%s/fakevolume", sysfs_path);
 
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
 	hotplug_event->type = HOTPLUG_EVENT_SYSFS_BLOCK;
 	g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
@@ -679,6 +714,7 @@ refresh_md_state (HalDevice *d)
         int num_components;
         gboolean ret;
         const char *sysfs_path;
+        const char *level;
 
         ret = FALSE;
 
@@ -689,49 +725,58 @@ refresh_md_state (HalDevice *d)
         }
 
         HAL_INFO (("In refresh_md_state() for '%s'", sysfs_path));
-
-        sync_action = hal_util_get_string_from_file (sysfs_path, "md/sync_action");
-        if (sync_action == NULL) {
-                HAL_WARNING (("Cannot get sync_action for %s", sysfs_path));
-                goto error;
-        }
-        if (strcmp (sync_action, "idle") == 0) {
+        level = hal_device_property_get_string (d, "storage.linux_raid.level");
+        HAL_INFO ((" MD Level is '%s'", level));
+		
+        /* MD linear device are not syncable */
+        if (strcmp (level, "linear") != 0) {
+	        sync_action = hal_util_get_string_from_file (sysfs_path, "md/sync_action");
+	        if (sync_action == NULL) {
+	                HAL_WARNING (("Cannot get sync_action for %s", sysfs_path));
+	                goto error;
+	        }
+	        if (strcmp (sync_action, "idle") == 0) {
+	                hal_device_property_set_bool (d, "storage.linux_raid.is_syncing", FALSE);
+			hal_device_property_remove (d, "storage.linux_raid.sync.action");
+			hal_device_property_remove (d, "storage.linux_raid.sync.speed");
+			hal_device_property_remove (d, "storage.linux_raid.sync.progress");
+	        } else {
+	                int speed;
+	                char *str_completed;
+                        
+	                hal_device_property_set_bool (d, "storage.linux_raid.is_syncing", TRUE);
+                        
+	                hal_device_property_set_string (d, "storage.linux_raid.sync.action", sync_action);
+                        
+			if (!hal_util_get_int_from_file (sysfs_path, "md/sync_speed", &speed, 10)) {
+	                        HAL_WARNING (("Cannot get sync_speed for %s", sysfs_path));
+	                } else {
+	                        hal_device_property_set_uint64 (d, "storage.linux_raid.sync.speed", speed);
+	                }
+                        
+	                if ((str_completed = hal_util_get_string_from_file (sysfs_path, "md/sync_completed")) == NULL) {
+	                        HAL_WARNING (("Cannot get sync_completed for %s", sysfs_path));
+	                } else {
+	                        long long int sync_pos, sync_total;
+                                
+	                        if (sscanf (str_completed, "%lld / %lld", &sync_pos, &sync_total) != 2) {
+	                                HAL_WARNING (("Malformed sync_completed '%s'", str_completed));
+	                        } else {
+	                                double sync_progress;
+	                                sync_progress = ((double) sync_pos) / ((double) sync_total);
+	                                hal_device_property_set_double (d, "storage.linux_raid.sync.progress", sync_progress);
+	                        }
+	                }
+                        
+	                /* check again in two seconds */
+#ifdef HAVE_GLIB_2_14
+	                g_timeout_add_seconds (2, md_check_sync_timeout, g_strdup (sysfs_path));
+#else
+	                g_timeout_add (2000, md_check_sync_timeout, g_strdup (sysfs_path));
+#endif
+	        }
+        } else
                 hal_device_property_set_bool (d, "storage.linux_raid.is_syncing", FALSE);
-		hal_device_property_remove (d, "storage.linux_raid.sync.action");
-		hal_device_property_remove (d, "storage.linux_raid.sync.speed");
-		hal_device_property_remove (d, "storage.linux_raid.sync.progress");
-        } else {
-                int speed;
-                char *str_completed;
-
-                hal_device_property_set_bool (d, "storage.linux_raid.is_syncing", TRUE);
-
-                hal_device_property_set_string (d, "storage.linux_raid.sync.action", sync_action);
-
-		if (!hal_util_get_int_from_file (sysfs_path, "md/sync_speed", &speed, 10)) {
-                        HAL_WARNING (("Cannot get sync_speed for %s", sysfs_path));
-                } else {
-                        hal_device_property_set_uint64 (d, "storage.linux_raid.sync.speed", speed);
-                }
-
-
-                if ((str_completed = hal_util_get_string_from_file (sysfs_path, "md/sync_completed")) == NULL) {
-                        HAL_WARNING (("Cannot get sync_completed for %s", sysfs_path));
-                } else {
-                        long long int sync_pos, sync_total;
-
-                        if (sscanf (str_completed, "%lld / %lld", &sync_pos, &sync_total) != 2) {
-                                HAL_WARNING (("Malformed sync_completed '%s'", str_completed));
-                        } else {
-                                double sync_progress;
-                                sync_progress = ((double) sync_pos) / ((double) sync_total);
-                                hal_device_property_set_double (d, "storage.linux_raid.sync.progress", sync_progress);
-                        }
-                }
-
-                /* check again in two seconds */
-                g_timeout_add (2000, md_check_sync_timeout, g_strdup (sysfs_path));
-        }
         
         if (!hal_util_get_int_from_file (sysfs_path, "md/raid_disks", &num_components, 0)) {
                 HAL_WARNING (("Cannot get number of RAID components"));
@@ -846,11 +891,13 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	int floppy_num;
 	gboolean is_device_mapper;
         gboolean is_md_device;
+	gboolean is_cciss_device;
         int md_number;
 
 	is_device_mapper = FALSE;
         is_fakevolume = FALSE;
         is_md_device = FALSE;
+	is_cciss_device = FALSE;
 
 	HAL_INFO (("block_add: sysfs_path=%s dev=%s is_part=%d, parent=0x%08x", 
 		   sysfs_path, device_file, is_partition, parent));
@@ -871,7 +918,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
                 /* set parent to root computer device object */
                 parent = hal_device_store_find (hald_get_gdl (), "/org/freedesktop/Hal/devices/computer");
                 if (parent == NULL)
-                        d = hal_device_store_find (hald_get_tdl (), "/org/freedesktop/Hal/devices/computer");
+                        parent = hal_device_store_find (hald_get_tdl (), "/org/freedesktop/Hal/devices/computer");
 	} else {
 		sysfs_path_real = g_strdup (sysfs_path);
 	}
@@ -887,61 +934,79 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 
 	d = hal_device_new ();
 
-	/* OK, no parent... it might a device-mapper device => check slaves/ subdir in sysfs */
+	/* OK, no parent... */
 	if (parent == NULL && !is_partition && !is_fakevolume && !hotplug_event->reposted) {
-		DIR * dir;
-		struct dirent *dp;
 		char path[HAL_PATH_MAX];
-
 		g_snprintf (path, HAL_PATH_MAX, "%s/slaves", sysfs_path);
-		HAL_INFO (("Looking in %s for Device Mapper", path));
 
-		if ((dir = opendir (path)) == NULL) {
-			HAL_WARNING (("Unable to open %s: %s", path, strerror(errno)));
+		if (device_file && (strstr(device_file, "/dev/cciss/") != NULL)) {
+			/* ... it's a cciss (HP Smart Array CCISS) device */
+			if (g_file_test (path, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
+				HAL_DEBUG(("block_add: found cciss a base device"));
+
+				is_cciss_device = TRUE;
+                		parent = hal_device_store_find (hald_get_gdl (), "/org/freedesktop/Hal/devices/computer");
+		                if (parent == NULL)
+                		        parent = hal_device_store_find (hald_get_tdl (), "/org/freedesktop/Hal/devices/computer");
+			} 
 		} else {
-			while (((dp = readdir (dir)) != NULL) && (parent == NULL)) {
-				char *link;
-				char *target;
+			/* ... it might a device-mapper device => check slaves/ subdir in sysfs */
+			DIR * dir;
+			struct dirent *dp;
 
-				link = g_strdup_printf ("%s/%s", path, dp->d_name);
-				target = resolve_symlink (link);
-				HAL_INFO ((" %s -> %s", link, target));
+			HAL_INFO (("Looking in %s for Device Mapper", path));
 
-				if (target != NULL) {
-					HalDevice *slave_volume;
+			if ((dir = opendir (path)) == NULL) {
+				HAL_WARNING (("Unable to open %s: %s", path, strerror(errno)));
+			} else {
+				while (((dp = readdir (dir)) != NULL) && (parent == NULL)) {
+					char *link;
+					char *target;
 
-					slave_volume = hal_device_store_match_key_value_string (hald_get_gdl (),
-												"linux.sysfs_path", 
-												target);
-					if (slave_volume != NULL) {
-						const char *slave_volume_stordev_udi;
-						const char *slave_volume_fstype;
+					link = g_strdup_printf ("%s/%s", path, dp->d_name);
+					target = resolve_symlink (link);
+					HAL_INFO ((" %s -> %s", link, target));
+					g_free (link);
 
-						slave_volume_stordev_udi = hal_device_property_get_string (slave_volume, "block.storage_device");
-						slave_volume_fstype = hal_device_property_get_string (slave_volume, "volume.fstype");
+					if (target != NULL) {
+						HalDevice *slave_volume;
 
-						/* Yup, we only support crypto_LUKS right now.
-						 *
-						 * In the future we can support other device-mapper mappings
-						 * such as LVM etc.
-						 */ 
-						if (slave_volume_stordev_udi != NULL &&
-						    slave_volume_fstype != NULL &&
-						    (strcmp (slave_volume_fstype, "crypto_LUKS") == 0)) {
-							HAL_INFO ((" slave_volume_stordev_udi='%s'!", slave_volume_stordev_udi));
-							parent = hal_device_store_find (hald_get_gdl (), slave_volume_stordev_udi);
-							if (parent != NULL) {
-								HAL_INFO ((" parent='%s'!", hal_device_get_udi (parent)));
-								hal_device_property_set_string (d, "volume.crypto_luks.clear.backing_volume", hal_device_get_udi (slave_volume));
-								is_device_mapper = TRUE;
+						slave_volume = hal_device_store_match_key_value_string (hald_get_gdl (),
+													"linux.sysfs_path", 
+													target);
+						if (slave_volume != NULL) {
+							const char *slave_volume_stordev_udi;
+							const char *slave_volume_fstype;
+
+							slave_volume_stordev_udi = hal_device_property_get_string (slave_volume, "block.storage_device");
+							slave_volume_fstype = hal_device_property_get_string (slave_volume, "volume.fstype");
+
+							/* Yup, we only support crypto_LUKS right now.
+							 *
+							 * In the future we can support other device-mapper mappings
+							 * such as LVM etc.
+							 */ 
+							if (slave_volume_stordev_udi != NULL &&
+							    slave_volume_fstype != NULL &&
+							    (strcmp (slave_volume_fstype, "crypto_LUKS") == 0)) {
+								HAL_INFO ((" slave_volume_stordev_udi='%s'!", slave_volume_stordev_udi));
+								parent = hal_device_store_find (hald_get_gdl (), slave_volume_stordev_udi);
+								if (parent != NULL) {
+									HAL_INFO ((" parent='%s'!", hal_device_get_udi (parent)));
+									hal_device_property_set_string (d, "volume.crypto_luks.clear.backing_volume", 
+													hal_device_get_udi (slave_volume));
+									is_device_mapper = TRUE;
+								}
 							}
+						} else {
+							HAL_INFO(("Couldn't find slave volume in devices"));
 						}
 					}
+					g_free (target);
 				}
-				g_free (target);
+				closedir(dir);
+				HAL_INFO (("Done looking in %s", path));
 			}
-			closedir(dir);
-			HAL_INFO (("Done looking in %s", path));
 		}
 
 	}
@@ -989,7 +1054,6 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		hal_device_property_set_string (d, "info.product", "PC Floppy Drive");
 		hal_device_property_set_string (d, "storage.drive_type", "floppy");
 		hal_device_property_set_string (d, "storage.originating_device", hal_device_get_udi (parent));
-		hal_device_property_set_string (d, "storage.physical_device", hal_device_get_udi (parent));
 		hal_device_property_set_bool (d, "storage.removable", TRUE);
 		hal_device_property_set_bool (d, "storage.removable.media_available", FALSE);
 		hal_device_property_set_bool (d, "storage.hotpluggable", FALSE);
@@ -1117,12 +1181,17 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
                                                 is_hotpluggable = hal_device_property_get_bool (
                                                         d, "storage.hotpluggable");
 
-                                        }
+					} else if (is_cciss_device) {
+						HAL_DEBUG (("block_add: parent=/org/freedesktop/Hal/devices/computer, is_cciss_device=true"));
+						hal_device_property_set_string (d, "storage.bus", "cciss");
+					}
                                         break;
                         }
 
 			/* Check info.subsystem */
 			if ((bus = hal_device_property_get_string (d_it, "info.subsystem")) != NULL) {
+				HAL_DEBUG(("block_add: info.subsystem='%s'", bus));
+
 				if (strcmp (bus, "scsi") == 0) {
 					scsidev = d_it;
 					physdev = d_it;
@@ -1161,12 +1230,25 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 					is_hotpluggable = TRUE;
 					hal_device_property_set_string (d, "storage.bus", "mmc");
 					break;
+				} else if (strcmp (bus, "memstick") == 0) {
+					physdev = d_it;
+					physdev_udi = udi_it;
+					is_hotpluggable = TRUE;
+					hal_device_property_set_string (d, "storage.bus", "memstick");
+					break;
 				} else if (strcmp (bus, "ccw") == 0) {
 					physdev = d_it;
 					physdev_udi = udi_it;
 					is_hotpluggable = TRUE;
-					hal_device_property_set_string
-						(d, "storage.bus", "ccw");
+					hal_device_property_set_string (d, "storage.bus", "ccw");
+				} else if (strcmp (bus, "vio") == 0) {
+					physdev = d_it;
+					physdev_udi = udi_it;
+					hal_device_property_set_string (d, "storage.bus", "vio");
+				} else if (strcmp (bus, "pci") == 0) {
+					physdev = d_it;
+					physdev_udi = udi_it;
+					hal_device_property_set_string (d, "storage.bus", "pci");
 				}
 			}
 
@@ -1181,7 +1263,6 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		}
 
 		hal_device_property_set_string (d, "storage.originating_device", physdev_udi);
-		hal_device_property_set_string (d, "storage.physical_device", physdev_udi);
 
 		if (!hal_util_get_int_from_file (sysfs_path_real, "removable", (gint *) &is_removable, 10)) {
 			HAL_WARNING (("Cannot get 'removable' file"));
@@ -1302,6 +1383,47 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 
 		} else if (strcmp (parent_bus, "mmc") == 0) {
 			hal_device_property_set_string (d, "storage.drive_type", "sd_mmc");
+		} else if (strcmp (parent_bus, "memstick") == 0) {
+			hal_device_property_set_string (d, "storage.drive_type", "memstick");
+		} else if (strcmp (parent_bus, "vio") == 0) {
+			char buf[256];
+			const gchar *prop;
+
+			snprintf(buf, sizeof(buf), "%s/device", sysfs_path_real);
+			prop = hal_util_get_string_from_file (buf, "name");
+			if (prop) {
+				if (strcmp (prop, "viocd") == 0) {
+					hal_device_property_set_string (d, "info.product", "Vio Virtual CD");
+					hal_device_property_set_string (d, "storage.drive_type", "cdrom");
+				} else if (strcmp (prop, "viodasd") == 0) {
+					hal_device_property_set_string (d, "info.product", "Vio Virtual Disk");
+					hal_device_property_set_string (d, "storage.drive_type", "disk");
+				} else if (strcmp (prop, "viotape") == 0) {
+					hal_device_property_set_string (d, "info.product", "Vio Virtual Tape");
+					hal_device_property_set_string (d, "storage.drive_type", "tape");
+				}
+			}
+		}
+		else if (strcmp (parent_bus, "xen") == 0) {
+			char buf[256];
+			gchar *media;
+
+			snprintf (buf, sizeof (buf), "%s/%s", sysfs_path_real, "device");
+			if ((media = hal_util_get_string_from_file (buf, "media")) != NULL) {
+				if (strcmp (media, "cdrom") == 0) {
+					hal_device_property_set_string (d, "info.product", "Xen Virtual CD");
+					hal_device_property_set_string (d, "storage.drive_type", media);
+					hal_device_property_set_bool (d, "storage.media_check_enabled", FALSE);
+				} else if (strcmp (media, "floppy") == 0) {
+					hal_device_property_set_string (d, "info.product", "Xen Virtual Floppy");
+					hal_device_property_set_string (d, "storage.drive_type", media);
+					hal_device_property_set_bool (d, "storage.media_check_enabled", FALSE);
+				}
+				else {
+					hal_device_property_set_string (d, "info.product", "Xen Virtual Disk");
+					hal_device_property_set_string (d, "storage.drive_type", "disk");
+				}
+			}
 		}
 
 		hal_device_property_set_string (d, "info.category", "storage");
@@ -1334,6 +1456,8 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	} else {
 		guint sysfs_path_len;
 		gboolean is_physical_partition;
+		char *volume_label;
+		char buf[64];
 
 		/*************************
 		 *
@@ -1343,20 +1467,46 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		hal_device_property_set_string (d, "block.storage_device", hal_device_get_udi (parent));
 
 		/* defaults */
-		hal_device_property_set_string (d, "storage.model", "");
 		hal_device_property_set_string (d, "volume.fstype", "");
 		hal_device_property_set_string (d, "volume.fsusage", "");
 		hal_device_property_set_string (d, "volume.fsversion", "");
 		hal_device_property_set_string (d, "volume.uuid", "");
 		hal_device_property_set_string (d, "volume.label", "");
 		hal_device_property_set_string (d, "volume.mount_point", "");
+
+		/* persistent properties from udev (may be empty) */
+                hal_device_property_set_string (d, "volume.fsusage", hotplug_event->sysfs.fsusage);
+                hal_device_property_set_string (d, "volume.fsversion", hotplug_event->sysfs.fsversion);
+                hal_device_property_set_string (d, "volume.uuid", hotplug_event->sysfs.fsuuid);
+                hal_device_property_set_string (d, "volume.fstype", hotplug_event->sysfs.fstype);
+		if (hotplug_event->sysfs.fstype[0] != '\0') {
+			snprintf (buf, sizeof (buf), "Volume (%s)", hotplug_event->sysfs.fstype);
+		} else {
+			snprintf (buf, sizeof (buf), "Volume");
+		}
+		hal_device_property_set_string (d, "info.product", buf);
+
+		volume_label = strdup_valid_utf8 (hotplug_event->sysfs.fslabel);
+		if (volume_label) {
+	                hal_device_property_set_string (d, "volume.label", volume_label);
+			if (volume_label[0] != '\0') {
+	                	hal_device_property_set_string (d, "info.product", volume_label);
+			}
+			g_free(volume_label);
+		}
+
 		hal_device_property_set_bool (d, "volume.is_mounted", FALSE);
 		hal_device_property_set_bool (d, "volume.is_mounted_read_only", FALSE);
 		hal_device_property_set_bool (d, "volume.linux.is_device_mapper", is_device_mapper);
-		hal_device_property_set_bool (
-			d, "volume.is_disc", 
-			strcmp (hal_device_property_get_string (parent, "storage.drive_type"), "cdrom") == 0);
-
+		/* Don't assume that the parent has storage capability, eg
+		 * if we are an MD partition then this is the case as we were
+		 * re-parented to the root computer device object earlier.
+		 */
+		if (hal_device_has_property(parent, "storage.drive_type")) {
+			hal_device_property_set_bool (d, "volume.is_disc", strcmp (hal_device_property_get_string (parent, "storage.drive_type"), "cdrom") == 0);
+		} else {
+			hal_device_property_set_bool (d, "volume.is_disc", FALSE);
+		}
 
 		is_physical_partition = TRUE;
 		if (is_fakevolume || is_device_mapper)
@@ -1365,8 +1515,10 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		hal_device_property_set_bool (d, "volume.is_partition", is_physical_partition);
 
 		hal_device_property_set_string (d, "info.category", "volume");
-		if (strcmp(hal_device_property_get_string (parent, "storage.drive_type"), "cdrom") == 0) {
-			hal_device_add_capability (d, "volume.disc");
+		if (hal_device_has_property(parent, "storage.drive_type")) {
+			if (strcmp(hal_device_property_get_string (parent, "storage.drive_type"), "cdrom") == 0) {
+				hal_device_add_capability (d, "volume.disc");
+			}
 		}
 		hal_device_add_capability (d, "volume");
 		hal_device_add_capability (d, "block");
@@ -1397,13 +1549,13 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		 * (block size requires opening the device file)
 		 */
 		hal_device_property_set_int (d, "volume.block_size", 512);
-		if (!hal_util_set_int_from_file (d, "volume.num_blocks", sysfs_path_real, "size", 0)) {
+		if (!hal_util_set_uint64_from_file (d, "volume.num_blocks", sysfs_path_real, "size", 0)) {
 			HAL_INFO (("Ignoring hotplug event - cannot read 'size'"));
 			goto error;
 		}
 		hal_device_property_set_uint64 (
 			d, "volume.size",
-			((dbus_uint64_t)(512)) * ((dbus_uint64_t)(hal_device_property_get_int (d, "volume.num_blocks"))));
+			((dbus_uint64_t)(512)) * ((dbus_uint64_t)(hal_device_property_get_uint64 (d, "volume.num_blocks"))));
 		/* TODO: move to prober? */
 		if (is_physical_partition) {
 			guint64 start_block;
@@ -1667,7 +1819,7 @@ blockdev_generate_add_hotplug_event (HalDevice *d)
 	serial      = hal_device_property_get_string (d, "storage.serial");
 	revision    = hal_device_property_get_string (d, "storage.firmware_revision");
 
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
 	hotplug_event->type = HOTPLUG_EVENT_SYSFS;
 	g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
@@ -1692,7 +1844,7 @@ blockdev_generate_remove_hotplug_event (HalDevice *d)
 
 	sysfs_path = hal_device_property_get_string (d, "linux.sysfs_path");
 
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->action = HOTPLUG_ACTION_REMOVE;
 	hotplug_event->type = HOTPLUG_EVENT_SYSFS;
 	g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
@@ -1711,22 +1863,22 @@ udev_get_device_file_for_sysfs_path (const char *sysfs_path)
         char *ret;
         char *u_stdout;
         int u_exit_status;
-        const char *argv[] = {"/usr/bin/udevinfo", "--root", "--query", "name", "--path", NULL, NULL};
+        const char *argv[] = {"/sbin/udevadm", "info", "--root", "--query", "name", "--path", NULL, NULL};
         GError *g_error;
 
         ret = NULL;
-        argv[5] = sysfs_path;
+        argv[6] = sysfs_path;
 
         g_error = NULL;
 
         if (!g_spawn_sync("/", 
                           (char **) argv, 
-                          NULL,           /* envp */
-                          0,              /* flags */
-                          NULL,           /* child_setup */
-                          NULL,           /* user_data */
+                          NULL,           			/* envp */
+                          G_SPAWN_LEAVE_DESCRIPTORS_OPEN, 	/* flags */
+                          NULL,           			/* child_setup */
+                          NULL,           			/* user_data */
                           &u_stdout,
-                          NULL,           /* stderr */
+                          NULL,           			/* stderr */
                           &u_exit_status,
                           &g_error)) {
                 HAL_ERROR (("Error spawning udevinfo: %s", g_error->message));
@@ -1820,7 +1972,7 @@ blockdev_process_mdstat (void)
                         } else {
                                 HAL_INFO (("Adding md device at '%s' ('%s')", sysfs_path, device_file));
 
-                                hotplug_event = g_new0 (HotplugEvent, 1);
+                                hotplug_event = g_slice_new0 (HotplugEvent);
                                 hotplug_event->action = HOTPLUG_ACTION_ADD;
                                 hotplug_event->type = HOTPLUG_EVENT_SYSFS_BLOCK;
                                 g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
@@ -1873,7 +2025,7 @@ blockdev_process_mdstat (void)
                                 
                                 HAL_INFO (("Removing md device at '%s' ('%s')", sysfs_path, device_file));
 
-                                hotplug_event = g_new0 (HotplugEvent, 1);
+                                hotplug_event = g_slice_new0 (HotplugEvent);
                                 hotplug_event->action = HOTPLUG_ACTION_REMOVE;
                                 hotplug_event->type = HOTPLUG_EVENT_SYSFS_BLOCK;
                                 g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
