@@ -30,6 +30,7 @@
 #endif
 
 #define _GNU_SOURCE 1
+#define DMI_SYSFS_PATH "/sys/class/dmi/id"
 
 #include <ctype.h>
 #include <errno.h>
@@ -54,6 +55,7 @@
 #include "../logger.h"
 #include "../osspec.h"
 #include "../util.h"
+#include "../util_helper.h"
 #include "../ids.h"
 
 #include "acpi.h"
@@ -120,13 +122,13 @@ hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
 		goto out;
 	}
 
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->type = HOTPLUG_EVENT_SYSFS;
 
 	while (bufpos < sizeof (buf)) {
 		size_t keylen;
 		char *key;
-		char *str;
+		char *str, *dstr;
 
 		key = &buf[bufpos];
 		keylen = strlen(key);
@@ -204,11 +206,15 @@ hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
 				g_strlcpy (hotplug_event->sysfs.fsuuid, str, sizeof(hotplug_event->sysfs.fsuuid));
 				g_free (str);
 			}
-		} else if (strncmp(key, "ID_FS_LABEL=", 12) == 0) {
-			if ((str = hal_util_strdup_valid_utf8(&key[12])) != NULL ) {
+		} else if (strncmp(key, "ID_FS_LABEL_ENC=", 16) == 0) {
+			dstr = g_malloc0 (keylen - 15);
+			hal_util_decode_escape (&key[16], dstr, sizeof(hotplug_event->sysfs.fslabel));
+
+			if ((str = hal_util_strdup_valid_utf8(dstr)) != NULL ) {
 				g_strlcpy (hotplug_event->sysfs.fslabel, str, sizeof(hotplug_event->sysfs.fslabel));
 				g_free (str);
 			}
+			g_free (dstr);
 		}
 	}
 
@@ -216,49 +222,68 @@ hald_udev_data (GIOChannel *source, GIOCondition condition, gpointer user_data)
 		HAL_INFO (("missing ACTION"));
 		goto invalid;
 	}
-	if (hotplug_event->sysfs.sysfs_path == NULL) {
+	if (hotplug_event->sysfs.sysfs_path[0] == '\0') {
 		HAL_INFO (("missing DEVPATH"));
 		goto invalid;
 	}
-	if (hotplug_event->sysfs.subsystem == NULL) {
-		HAL_INFO (("missing SUSBSYSTEM"));
+	if (hotplug_event->sysfs.subsystem[0] == '\0') {
+		HAL_INFO (("missing SUBSYSTEM"));
 		goto invalid;
 	}
 
+	/* This is a workaround for temporary cryptsetup devices, HAL should ignore them. 
+	 * There is already a fix for this issue in the udev git master as soon as a new
+	 * udev release is available and HAL starts to depend on the new version we should 
+	 * remove this again. (added 2008-03-04)
+	 */
+	if (strncmp (hotplug_event->sysfs.device_file, "/dev/mapper/temporary-cryptsetup-", 33) == 0) {
+		HAL_INFO (("Temporary workaround: ignoring temporary cryptsetup file"));
+		goto invalid;
+	}
+	if (strstr (hotplug_event->sysfs.device_file, "/dm-") != NULL) {
+		HAL_DEBUG (("Found a dm-device (%s), mark it", hotplug_event->sysfs.device_file));
+		hotplug_event->sysfs.is_dm_device = TRUE;
+	}
+		
+
 	HAL_INFO (("SEQNUM=%lld, ACTION=%s, SUBSYSTEM=%s, DEVPATH=%s, DEVNAME=%s, IFINDEX=%d",
 		   hotplug_event->sysfs.seqnum, action, hotplug_event->sysfs.subsystem, hotplug_event->sysfs.sysfs_path,
 		   hotplug_event->sysfs.device_file, hotplug_event->sysfs.net_ifindex));
 
-	if (strcmp (action, "add") == 0) {
-		hotplug_event->action = HOTPLUG_ACTION_ADD;
-		hotplug_event_enqueue (hotplug_event);
-		hotplug_event_process_queue ();
-		goto out;
-	}
+	/* ignore module and driver events, until we really need them */
+	if ((strcmp (hotplug_event->sysfs.subsystem, "drivers") != 0) && 
+	    (strcmp (hotplug_event->sysfs.subsystem, "module") != 0)) {
+		if (strcmp (action, "add") == 0) {
+			hotplug_event->action = HOTPLUG_ACTION_ADD;
+			hotplug_event_enqueue (hotplug_event);
+			hotplug_event_process_queue ();
+			goto out;
+		}
 
-	if (strcmp (action, "change") == 0) {
-		hotplug_event->action = HOTPLUG_ACTION_CHANGE;
-		hotplug_event_enqueue (hotplug_event);
-		hotplug_event_process_queue ();
-		goto out;
-	}
+		if (strcmp (action, "change") == 0) {
+			hotplug_event->action = HOTPLUG_ACTION_CHANGE;
+			hotplug_event_enqueue (hotplug_event);
+			hotplug_event_process_queue ();
+			goto out;
+		}
 
-	if (strcmp (action, "move") == 0) {
-		hotplug_event->action = HOTPLUG_ACTION_MOVE;
-		hotplug_event_enqueue (hotplug_event);
-		hotplug_event_process_queue ();
-		goto out;
-	}
+		if (strcmp (action, "move") == 0) {
+			hotplug_event->action = HOTPLUG_ACTION_MOVE;
+			hotplug_event_enqueue (hotplug_event);
+			hotplug_event_process_queue ();
+			goto out;
+		}
 
-	if (strcmp (action, "remove") == 0) {
-		hotplug_event->action = HOTPLUG_ACTION_REMOVE;
-		hotplug_event_enqueue (hotplug_event);
-		hotplug_event_process_queue ();
-		goto out;
+		if (strcmp (action, "remove") == 0) {
+			hotplug_event->action = HOTPLUG_ACTION_REMOVE;
+			hotplug_event_enqueue (hotplug_event);
+			hotplug_event_process_queue ();
+			goto out;
+		}
 	}
 
 invalid:
-	g_free (hotplug_event);
+	g_slice_free (HotplugEvent, hotplug_event);
 
 out:
 	return TRUE;
@@ -304,6 +329,42 @@ GIOChannel *get_mdstat_channel (void)
         return mdstat_channel;
 }
 
+/* 
+ * NOTE: We need to use this function to parse if HAL is privileged 
+ *       since some of the dmi keys in sysfs are only readable by root
+ */
+static void 
+osspec_privileged_init_preparse_set_dmi (gboolean set, HalDevice *d) 
+{
+	gchar *buf;
+	static char *product_serial;
+	static char *product_uuid;
+	static char *board_serial;
+	static gboolean parsed = FALSE; 
+
+	if (g_file_test (DMI_SYSFS_PATH, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
+
+		if (!set){
+			if ((buf = hal_util_get_string_from_file(DMI_SYSFS_PATH, "product_serial")) != NULL)
+				product_serial = g_strdup ( buf );
+			if ((buf = hal_util_get_string_from_file(DMI_SYSFS_PATH, "product_uuid")) != NULL)
+				product_uuid = g_strdup ( buf );
+			if ((buf = hal_util_get_string_from_file(DMI_SYSFS_PATH, "board_serial")) != NULL)
+				board_serial = g_strdup ( buf );
+
+			parsed = TRUE;
+		} else {
+			if (d != NULL && parsed) {
+				hal_device_property_set_string (d, "system.hardware.serial", product_serial);		
+				hal_device_property_set_string (d, "system.hardware.uuid", product_uuid);
+				hal_device_property_set_string (d, "system.board.serial", board_serial);
+				g_free (product_serial);
+				g_free (product_uuid);
+				parsed = FALSE;
+			}
+		}
+	}
+}
 
 void
 osspec_privileged_init (void)
@@ -332,6 +393,8 @@ osspec_privileged_init (void)
 	
 	if (err != NULL)
 		g_error_free (err);
+
+	osspec_privileged_init_preparse_set_dmi(FALSE, NULL);
 }
 
 void
@@ -347,6 +410,9 @@ osspec_init (void)
 	/*
 	 * setup socket for listening from messages from udev
 	 */
+
+	hal_device_store_index_property (hald_get_gdl (), "linux.sysfs_path");
+
 	memset(&saddr, 0x00, sizeof(saddr));
 	saddr.sun_family = AF_LOCAL;
 	/* use abstract namespace for socket path */
@@ -449,12 +515,91 @@ computer_probing_helper_done (HalDevice *d)
 	hal_util_callout_device_add (d, computer_callouts_add_done, NULL, NULL);
 }
 
+static void 
+computer_dmi_map (HalDevice *d, gboolean dmidecode) 
+{
+	/* Map the chassis type from dmidecode.c to a sensible type used in hal 
+	 *
+	 * See also 3.3.4.1 of the "System Management BIOS Reference Specification, 
+	 * Version 2.6.1" (Preliminary Standard) document, available from 
+	 * http://www.dmtf.org/standards/smbios.
+	 *
+	 * TODO: figure out WTF the mapping should be; "Lunch Box"? Give me a break :-)
+	 */
+	static const char *chassis_map[] = {
+		"Other",                 "unknown", /* 0x01 */
+		"Unknown",               "unknown",
+		"Desktop",               "desktop",
+		"Low Profile Desktop",   "desktop",
+		"Pizza Box",             "server",
+		"Mini Tower",            "desktop",
+		"Tower",                 "desktop",
+		"Portable",              "laptop",
+		"Laptop",                "laptop",
+		"Notebook",              "laptop",
+		"Hand Held",             "handheld",
+		"Docking Station",       "laptop",
+		"All In One",            "unknown",
+		"Sub Notebook",          "laptop",
+		"Space-saving",          "desktop",
+		"Lunch Box",             "unknown",
+		"Main Server Chassis",   "server",
+		"Expansion Chassis",     "unknown",
+		"Sub Chassis",           "unknown",
+		"Bus Expansion Chassis", "unknown",
+		"Peripheral Chassis",    "unknown",
+		"RAID Chassis",          "unknown",
+		"Rack Mount Chassis",    "unknown",
+		"Sealed-case PC",        "unknown",
+		"Multi-system",          "unknown",
+		"CompactPCI",		 "unknonw",
+		"AdvancedTCA",		 "unknown", 
+		"Blade",		 "server",
+		"Blade Enclosure"	 "unknown", /* 0x1D */
+		NULL
+	};
+
+
+	if (dmidecode) {
+		/* do mapping from text to text type */ 
+		unsigned int i;
+		const char *chassis_type;
+
+		/* now map the smbios.* properties to our generic system.formfactor property */
+		if ((chassis_type = hal_device_property_get_string (d, "system.chassis.type")) != NULL) {
+			
+			for (i = 0; chassis_map[i] != NULL; i += 2) {
+				if (strcmp (chassis_map[i], chassis_type) == 0) {
+					hal_device_property_set_string (d, "system.formfactor", chassis_map[i+1]);
+					break;
+				}
+			}
+		} 
+	} else {
+		gint chassis_type;
+
+		/* do mapping from integer type to text type*/
+		/* get the chassis type and map it to the related text info */
+		if (hal_util_get_int_from_file(DMI_SYSFS_PATH, "chassis_type", &chassis_type, 10)) {
+
+			if ((chassis_type > 0) && (chassis_type < 28) && (chassis_map[(chassis_type-1)*2] != NULL)) {
+				hal_device_property_set_string (d, "system.chassis.type", chassis_map[((chassis_type-1)*2)]);
+				hal_device_property_set_string (d, "system.formfactor", chassis_map[((chassis_type-1)*2)+1]);
+			}
+
+		} else {
+			hal_device_property_set_string (d, "system.chassis.type", "Unknown");
+			hal_device_property_set_string (d, "system.formfactor", "unknown");
+		}
+	}
+
+}
+
 static void 
 computer_probing_pcbios_helper_done (HalDevice *d, guint32 exit_type, 
 	                             gint return_code, gchar **error, 
 				     gpointer data1, gpointer data2)
 {
-	const char *chassis_type;
 	const char *system_manufacturer;
 	const char *system_product;
 	const char *system_version;
@@ -468,9 +613,6 @@ computer_probing_pcbios_helper_done (HalDevice *d, guint32 exit_type,
 	    (system_version = hal_device_property_get_string (d, "system.hardware.version")) != NULL) {
 		char buf[128];
 
-		/* depricated 2008-02-28 */
-		hal_device_property_set_string (d, "system.vendor", system_manufacturer);
-
 		if (strcmp(system_version, "Not Specified" ) != 0 ) {
 			g_snprintf (buf, sizeof (buf), "%s %s", system_product, system_version);
 			hal_device_property_set_string (d, "system.product", buf);
@@ -481,54 +623,7 @@ computer_probing_pcbios_helper_done (HalDevice *d, guint32 exit_type,
 
 
 	if (!hal_device_has_property (d, "system.formfactor")) {
-		/* now map the smbios.* properties to our generic system.formfactor property */
-		if ((chassis_type = hal_device_property_get_string (d, "smbios.chassis.type")) != NULL) {
-			unsigned int i;
-
-			/* Map the chassis type from dmidecode.c to a sensible type used in hal 
-			 *
-			 * See also 3.3.4.1 of the "System Management BIOS Reference Specification, 
-			 * Version 2.3.4" document, available from http://www.dmtf.org/standards/smbios.
-			 *
-			 * TODO: figure out WTF the mapping should be; "Lunch Box"? Give me a break :-)
-			 */
-			static const char *chassis_map[] = {
-				"Other",                 "unknown",
-				"Unknown",               "unknown",
-				"Desktop",               "desktop",
-				"Low Profile Desktop",   "desktop",
-				"Pizza Box",             "server",
-				"Mini Tower",            "desktop",
-				"Tower",                 "desktop",
-				"Portable",              "laptop",
-				"Laptop",                "laptop",
-				"Notebook",              "laptop",
-				"Hand Held",             "handheld",
-				"Docking Station",       "laptop",
-				"All In One",            "unknown",
-				"Sub Notebook",          "laptop",
-				"Space-saving",          "unknown",
-				"Lunch Box",             "unknown",
-				"Main Server Chassis",   "server",
-				"Expansion Chassis",     "unknown",
-				"Sub Chassis",           "unknown",
-				"Bus Expansion Chassis", "unknown",
-				"Peripheral Chassis",    "unknown",
-				"RAID Chassis",          "unknown",
-				"Rack Mount Chassis",    "unknown",
-				"Sealed-case PC",        "unknown",
-				"Multi-system",          "unknown",
-				NULL
-			};
-			
-			for (i = 0; chassis_map[i] != NULL; i += 2) {
-				if (strcmp (chassis_map[i], chassis_type) == 0) {
-					hal_device_property_set_string (d, "system.formfactor", chassis_map[i+1]);
-					break;
-				}
-			}
-		       
-		} 
+		computer_dmi_map (d, TRUE);
 	}
 out:
 	computer_probing_helper_done (d);
@@ -537,27 +632,16 @@ out:
 static void
 set_suspend_hibernate_keys (HalDevice *d)
 {
-	int can_suspend;
-	int can_hibernate;
-	ssize_t read;
-	size_t len;
+	gboolean can_suspend;
+	gboolean can_hibernate;
 	char *poweroptions;
 	const char *pmtype;
-	FILE *fp;
 
 	can_suspend = FALSE;
 	can_hibernate = FALSE;
 
 	/* try to find 'mem' and 'disk' in /sys/power/state */
-	fp = fopen ("/sys/power/state", "r");
-	if (fp == NULL) {
-		HAL_WARNING (("Could not open /sys/power/state"));
-		goto out;
-	}
-	poweroptions = NULL;
-	len = 0;
-	read = getline (&poweroptions, &len, fp);
-	fclose (fp);
+	poweroptions = hal_util_get_string_from_file("/sys/power/", "state");
 	if (poweroptions == NULL) {
 		HAL_WARNING (("Contents of /sys/power/state invalid"));
 		goto out;
@@ -566,31 +650,29 @@ set_suspend_hibernate_keys (HalDevice *d)
 		can_suspend = TRUE;
 	if (strstr (poweroptions, "disk"))
 		can_hibernate = TRUE;
-	free (poweroptions);
 
-	pmtype = hal_device_property_get_string (d, "power_management.type");
-	if (pmtype != NULL && strcmp(pmtype, "pmu") == 0) {
-		/* We got our own helper for suspend PMU machines */
-		can_suspend = TRUE;
+	if (!can_suspend) {
+		pmtype = hal_device_property_get_string (d, "power_management.type");
+		if (pmtype != NULL && strcmp(pmtype, "pmu") == 0) {
+			/* We got our own helper for suspend PMU machines */
+			can_suspend = TRUE;
+		}
 	}
 
-	/* check for the presence of suspend2 */
-	if (access ("/proc/software_suspend", F_OK) == 0)
-		can_hibernate = TRUE;
-	if (access ("/proc/suspend2", F_OK) == 0)
-		can_hibernate = TRUE;
-	if (access ("/sys/power/suspend2/version", F_OK) == 0)
-		can_hibernate = TRUE;
+	if (!can_hibernate) {
+		/* check for the presence of suspend2 */
+		if (access ("/proc/software_suspend", F_OK) == 0) {
+			can_hibernate = TRUE;
+		} else if (access ("/proc/suspend2", F_OK) == 0) {
+			can_hibernate = TRUE;
+		} else if (access ("/sys/power/suspend2/version", F_OK) == 0) {
+			can_hibernate = TRUE;
+		}
+	}
 out:
 	hal_device_property_set_bool (d, "power_management.can_suspend", can_suspend);
 	hal_device_property_set_bool (d, "power_management.can_suspend_hybrid", FALSE);
 	hal_device_property_set_bool (d, "power_management.can_hibernate", can_hibernate);
-
-	/* WARNING: These keys are depreciated and power_management.can_suspend
-	 * and power_management.can_hibernate should be used instead.
-	 * These properties will be removed, but not before May 1st 2007. */
-	hal_device_property_set_bool (d, "power_management.can_suspend_to_ram", can_suspend);
-	hal_device_property_set_bool (d, "power_management.can_suspend_to_disk", can_hibernate);
 }
 
 static void
@@ -645,35 +727,65 @@ detect_openfirmware_formfactor(HalDevice *root)
 	}
 }
 
-static void
-probe_openfirmware(HalDevice *root) 
+static gboolean
+decode_dmi_from_openfirmware (HalDevice *root)
 {
 #define DEVICE_TREE "/proc/device-tree/"
-	if (!g_file_test(DEVICE_TREE, G_FILE_TEST_IS_DIR)) {
-		return;
-	}
+	if (!g_file_test(DEVICE_TREE, G_FILE_TEST_IS_DIR))
+		return FALSE;
+
 	get_openfirmware_entry(root, "openfirmware.model", 
 		DEVICE_TREE "model", FALSE);
 	get_openfirmware_entry(root, "openfirmware.compatible", 
 		DEVICE_TREE "compatible", TRUE);
 	detect_openfirmware_formfactor(root);
+	return TRUE;
+}
+
+static gboolean
+decode_dmi_from_sysfs (HalDevice *d)
+{
+	if (!g_file_test (DMI_SYSFS_PATH, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)))
+		return FALSE;
+
+	osspec_privileged_init_preparse_set_dmi(TRUE, d);
+	hal_util_set_string_from_file(d, "system.firmware.vendor", DMI_SYSFS_PATH, "bios_vendor");
+	hal_util_set_string_from_file(d, "system.firmware.version", DMI_SYSFS_PATH, "bios_version");
+	hal_util_set_string_from_file(d, "system.firmware.release_date", DMI_SYSFS_PATH, "bios_date");
+	hal_util_set_string_from_file(d, "system.hardware.vendor", DMI_SYSFS_PATH, "sys_vendor");
+	hal_util_set_string_from_file(d, "system.hardware.product", DMI_SYSFS_PATH, "product_name");
+	hal_util_set_string_from_file(d, "system.hardware.version", DMI_SYSFS_PATH, "product_version");
+	hal_util_set_string_from_file(d, "system.chassis.manufacturer", DMI_SYSFS_PATH, "chassis_vendor");
+	hal_util_set_string_from_file(d, "system.board.product", DMI_SYSFS_PATH, "board_name");
+	hal_util_set_string_from_file(d, "system.board.version", DMI_SYSFS_PATH, "board_version");
+	hal_util_set_string_from_file(d, "system.board.vendor", DMI_SYSFS_PATH, "board_vendor");
+	computer_dmi_map (d, FALSE);
+
+	return TRUE;
 }
 
 static void
 decode_dmi (HalDevice *d)
 {
-        if (g_file_test ("/usr/sbin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
-            g_file_test ("/bin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
-            g_file_test ("/sbin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
-            g_file_test ("/usr/local/sbin//dmidecode", G_FILE_TEST_IS_EXECUTABLE)) {
-                hald_runner_run (d, "hald-probe-smbios", NULL, HAL_HELPER_TIMEOUT,
-                                 computer_probing_pcbios_helper_done, NULL, NULL);
-        } else {
-                /* no probing */
-                probe_openfirmware (d);
-                computer_probing_helper_done (d);
-        }
-
+	/* try to get the dmi infos from sysfs instead of call dmidecode*/
+	if (decode_dmi_from_sysfs(d) ||
+	    decode_dmi_from_openfirmware (d)) {
+		HAL_INFO (("got DMI from files"));
+		computer_probing_helper_done (d);
+	} else {
+		if (g_file_test ("/usr/sbin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
+		    g_file_test ("/bin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
+		    g_file_test ("/sbin/dmidecode", G_FILE_TEST_IS_EXECUTABLE) ||
+		    g_file_test ("/usr/local/sbin/dmidecode", G_FILE_TEST_IS_EXECUTABLE)) {
+			HAL_INFO (("getting DMI from prober"));
+			hald_runner_run (d, "hald-probe-smbios", NULL, HAL_HELPER_TIMEOUT,
+					 computer_probing_pcbios_helper_done, NULL, NULL);
+		} else {
+			/* no probing possible */
+			HAL_INFO (("failed to probe DMI"));
+			computer_probing_helper_done (d);
+		}
+	}
 }
 
 static void 
@@ -681,8 +793,8 @@ computer_probing_pm_is_supported_helper_done (HalDevice *d, guint32 exit_type,
                                               gint return_code, gchar **error, 
                                               gpointer data1, gpointer data2)
 {
-        HAL_INFO (("In computer_probing_pm_is_supported_helper_done"));
-        decode_dmi (d);
+	HAL_INFO (("In computer_probing_pm_is_supported_helper_done"));
+	decode_dmi (d);
 }
 
 static void
@@ -726,14 +838,34 @@ osspec_probe (void)
 	hald_runner_set_method_run_notify ((HaldRunnerRunNotify) hotplug_event_process_queue, NULL);
 	root = hal_device_new ();
 	hal_device_property_set_string (root, "info.subsystem", "unknown");
-	hal_device_property_set_string (root, "info.bus", "unknown");
 	hal_device_property_set_string (root, "info.product", "Computer");
-	hal_device_property_set_string (root, "info.udi", "/org/freedesktop/Hal/devices/computer");
 	hal_device_set_udi (root, "/org/freedesktop/Hal/devices/computer");
 
+	if (PACKAGE_VERSION) {
+		int major, minor, micro;
+
+		hal_device_property_set_string (root, "org.freedesktop.Hal.version", PACKAGE_VERSION);
+		if ( sscanf( PACKAGE_VERSION, "%d.%d.%d", &major, &minor, &micro ) == 3 ) {
+			hal_device_property_set_int (root, "org.freedesktop.Hal.version.major", major);
+                        hal_device_property_set_int (root, "org.freedesktop.Hal.version.minor", minor);
+                        hal_device_property_set_int (root, "org.freedesktop.Hal.version.micro", micro);
+		}
+	}
+
 	if (uname (&un) >= 0) {
 		hal_device_property_set_string (root, "system.kernel.name", un.sysname);
 		hal_device_property_set_string (root, "system.kernel.version", un.release);
+		if (un.release != NULL && un.release[0] != '\0') {
+                        int major, minor, micro ;
+
+			/* check if we can parse the major.minor.micro info and ignore the rest */
+                        if ( sscanf( un.release, "%d.%d.%d", &major, &minor, &micro ) >= 3 ) {
+				hal_device_property_set_int (root, "system.kernel.version.major", major);
+				hal_device_property_set_int (root, "system.kernel.version.minor", minor);
+				hal_device_property_set_int (root, "system.kernel.version.micro", micro);
+			}
+                }
+
 		hal_device_property_set_string (root, "system.kernel.machine", un.machine);
 	}
 
@@ -785,6 +917,11 @@ hal_util_get_driver_name (const char *sysfs_path, gchar *driver_name)
 	gchar driver_path[HAL_PATH_MAX];
 	struct stat statbuf;
 
+	if (sysfs_path == NULL) {
+		HAL_WARNING (("hal_util_get_driver_name: sysfs_path == NULL"));
+		return FALSE;
+	}
+
 	g_snprintf (driver_path, sizeof (driver_path), "%s/driver", sysfs_path);
 	if (stat (driver_path, &statbuf) == 0) {
 		gchar buf[256];
@@ -803,6 +940,11 @@ hal_util_set_driver (HalDevice *d, const char *property_name, const char *sysfs_
 	gboolean ret;
 	gchar driver_name[256];
 
+	if (d == NULL || property_name == NULL || sysfs_path == NULL) {
+		HAL_WARNING (("hal_util_set_driver: d, property_name or sysfs_path == NULL"));
+		return FALSE;
+	}
+
 	memset (driver_name, '\0', sizeof (driver_name));
 	ret = hal_util_get_driver_name (sysfs_path, driver_name);
 	if (ret == TRUE)
