@@ -32,6 +32,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <linux/kdev_t.h>
+#include <linux/hdreg.h>
 #include <linux/cdrom.h>
 #include <linux/fs.h>
 #include <mntent.h>
@@ -43,25 +44,14 @@
 #include <unistd.h>
 
 #include <glib.h>
-#include <libvolume_id.h>
+#include <blkid.h>
 
 #include "libhal/libhal.h"
 #include "partutil/partutil.h"
 #include "linux_dvd_rw_utils.h"
 
 #include "../../logger.h"
-
-static void vid_log(int priority, const char *file, int line, const char *format, ...)
-{
-	char log_str[1024];
-	va_list args;
-
-	va_start(args, format);
-	vsnprintf(log_str, sizeof(log_str), format, args);
-	logger_forward_debug("%s:%i %s\n", file, line, log_str);
-	va_end(args);
-}
-
+#include "../../util_helper.h"
 
 /** Check if a filesystem on a special device file is mounted
  *
@@ -75,7 +65,6 @@ is_mounted (const char *device_file)
 	FILE *f;
 	dbus_bool_t rc;
 	struct mntent mnt;
-	struct mntent *mnte;
 	char buf[512];
 
 	rc = FALSE;
@@ -83,7 +72,7 @@ is_mounted (const char *device_file)
 	if ((f = setmntent ("/etc/mtab", "r")) == NULL)
 		goto out;
 
-	while ((mnte = getmntent_r (f, &mnt, buf, sizeof(buf))) != NULL) {
+	while (getmntent_r (f, &mnt, buf, sizeof(buf)) != NULL) {
 		if (strcmp (device_file, mnt.mnt_fsname) == 0) {
 			rc = TRUE;
 			goto out1;
@@ -115,8 +104,7 @@ main (int argc, char *argv[])
 
 	fd = -1;
 
-	/* hook in our debug into libvolume_id */
-	volume_id_log_fn = vid_log;
+        fprintf (stderr, "woohoo\n");
 
 	/* assume failure */
 	ret = 1;
@@ -152,27 +140,70 @@ main (int argc, char *argv[])
 	HAL_DEBUG (("Doing probe-storage for %s (bus %s) (drive_type %s) (udi=%s) (--only-check-for-fs==%d)", 
 	     device_file, bus, drive_type, udi, only_check_for_fs));
 
-	if (!only_check_for_fs) {
-		/* Get properties for CD-ROM drive */
-		if (strcmp (drive_type, "cdrom") == 0) {
-			int capabilities;
-			int read_speed, write_speed;
-			char *write_speeds;
-			
-			HAL_DEBUG (("Doing open (\"%s\", O_RDONLY | O_NONBLOCK)", device_file));
-			fd = open (device_file, O_RDONLY | O_NONBLOCK);
-			if (fd < 0) {
-				HAL_ERROR (("Cannot open %s: %s", device_file, strerror (errno)));
-				goto out;
-			}
-			HAL_DEBUG (("Returned from open(2)"));
+	/* get model from SCSI/IDE layer and not via sysfs to prevent truncated strings (see bnc#394390) */
+	if (strcmp (bus, "scsi") == 0){
+		struct hd_driveid id;
+		gchar *model = NULL;
 
-			if (ioctl (fd, CDROM_SET_OPTIONS, CDO_USE_FFLAGS) < 0) {
-				HAL_ERROR (("Error: CDROM_SET_OPTIONS failed: %s\n", strerror(errno)));
-				close (fd);
-				goto out;
+		fd = open(device_file, O_RDONLY|O_NONBLOCK);
+		if (fd < 0) {
+			HAL_WARNING(("unable to open '%s', error: '%s'", device_file, strerror (errno)));
+		} else {
+			if (ioctl(fd, HDIO_GET_IDENTITY, &id)) {
+				HAL_WARNING (("HDIO_GET_IDENTITY unsupported or failed for: '%s' with error: '%s'", device_file, strerror (errno)));
+			} else {
+				char str[41];
+				size_t i, len;
+
+				i = 0;			
+				len = 40; 
+				
+				while (len && isspace(id.model[len-1]))
+			                len--;
+				
+				while (i < len) {
+					str[i] = id.model[i];
+					i++;
+				}
+				str[i] = '\0';
+
+				g_strstrip(str);
+	
+				if ((model = hal_util_strdup_valid_utf8(str)) != NULL) {
+					model = g_strdelimit(model, "/", '_');
+				}
 			}
-			
+			close(fd);
+		}
+		
+		if (model != NULL) {
+			libhal_changeset_set_property_string (cs, "storage.model", model);
+			libhal_changeset_set_property_string (cs, "info.product", model);
+			g_free (model);
+		}
+	}
+
+	/* Get properties for CD-ROM drive */
+	if (strcmp (drive_type, "cdrom") == 0) {
+		int capabilities;
+		int read_speed, write_speed;
+		char *write_speeds;
+		
+		HAL_DEBUG (("Doing open (\"%s\", O_RDONLY | O_NONBLOCK)", device_file));
+		fd = open (device_file, O_RDONLY | O_NONBLOCK);
+		if (fd < 0) {
+			HAL_ERROR (("Cannot open %s: %s", device_file, strerror (errno)));
+			goto out;
+		}
+		HAL_DEBUG (("Returned from open(2)"));
+
+		if (ioctl (fd, CDROM_SET_OPTIONS, CDO_USE_FFLAGS) < 0) {
+			HAL_ERROR (("Error: CDROM_SET_OPTIONS failed: %s\n", strerror(errno)));
+			close (fd);
+			goto out;
+		}
+		
+		if (!only_check_for_fs) {
 			capabilities = ioctl (fd, CDROM_GET_CAPABILITY, 0);
 			if (capabilities < 0) {
 				close (fd);
@@ -185,6 +216,7 @@ main (int argc, char *argv[])
 			libhal_changeset_set_property_bool (cs, "storage.cdrom.dvd", FALSE);
 			libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdr", FALSE);
 			libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdrw", FALSE);
+			libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdrdl", FALSE);
 			libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdram", FALSE);
 			libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdplusr", FALSE);
 			libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdplusrw", FALSE);
@@ -217,14 +249,28 @@ main (int argc, char *argv[])
 
 				if (profile & DRIVE_CDROM_CAPS_DVDRW)
 					libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdrw", TRUE);
+				if (profile & DRIVE_CDROM_CAPS_DVDRDL)
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdrdl", TRUE);
 				if (profile & DRIVE_CDROM_CAPS_DVDPLUSR)
-					libhal_changeset_set_property_bool(cs, "storage.cdrom.dvdplusr", TRUE);
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdplusr", TRUE);
 				if (profile & DRIVE_CDROM_CAPS_DVDPLUSRW)
 					libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdplusrw", TRUE);
 				if (profile & DRIVE_CDROM_CAPS_DVDPLUSRWDL)
 					libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdplusrwdl", TRUE);
 				if (profile & DRIVE_CDROM_CAPS_DVDPLUSRDL)
-                                        libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdplusrdl", TRUE);
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdplusrdl", TRUE);
+				if (profile & DRIVE_CDROM_CAPS_BDROM)
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.bd", TRUE);
+				if (profile & DRIVE_CDROM_CAPS_BDR)
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.bdr", TRUE);
+				if (profile & DRIVE_CDROM_CAPS_BDRE)
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.bdre", TRUE);
+				if (profile & DRIVE_CDROM_CAPS_HDDVDROM)
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.hddvd", TRUE);
+				if (profile & DRIVE_CDROM_CAPS_HDDVDR)
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.hddvdr", TRUE);
+				if (profile & DRIVE_CDROM_CAPS_HDDVDRW)
+					libhal_changeset_set_property_bool (cs, "storage.cdrom.hddvdrw", TRUE);
 			}
 			if (capabilities & CDC_DVD_R) {
 				libhal_changeset_set_property_bool (cs, "storage.cdrom.dvdr", TRUE);
@@ -253,36 +299,37 @@ main (int argc, char *argv[])
 			} else {
 				libhal_changeset_set_property_bool (cs, "storage.cdrom.support_multisession", FALSE);
 			}
-		
-			if (get_read_write_speed(fd, &read_speed, &write_speed, &write_speeds) >= 0) {
-				libhal_changeset_set_property_int (cs, "storage.cdrom.read_speed", read_speed);
-				if (write_speed > 0) {
-					libhal_changeset_set_property_int (
-						cs, "storage.cdrom.write_speed", write_speed);
-					
-					if (write_speeds != NULL)
-					{
-						gchar **wspeeds;
- 						wspeeds = g_strsplit_set (write_speeds, ",", -1);
-						libhal_changeset_set_property_strlist (cs, 
-										       "storage.cdrom.write_speeds", 
-										       (const char **) wspeeds);
-						g_strfreev (wspeeds);
-						free (write_speeds);
-					}
-				} else {
-					gchar *wspeeds[1] = {NULL};
-					libhal_changeset_set_property_int (cs, "storage.cdrom.write_speed", 0);
+	
+		}
+
+		if (get_read_write_speed(fd, &read_speed, &write_speed, &write_speeds) >= 0) {
+			libhal_changeset_set_property_int (cs, "storage.cdrom.read_speed", read_speed);
+
+			if (write_speed > 0) {
+				libhal_changeset_set_property_int (cs, "storage.cdrom.write_speed", write_speed);
+				
+				if (write_speeds != NULL)
+				{
+					gchar **wspeeds;
+
+					wspeeds = g_strsplit_set (write_speeds, ",", -1);
 					libhal_changeset_set_property_strlist (cs, "storage.cdrom.write_speeds", 
 									       (const char **) wspeeds);
+					g_strfreev (wspeeds);
+					free (write_speeds);
+				
 				}
+			} else {
+				gchar *wspeeds[1] = {NULL};
+				libhal_changeset_set_property_int (cs, "storage.cdrom.write_speed", 0);
+				libhal_changeset_set_property_strlist (cs, "storage.cdrom.write_speeds", 
+								       (const char **) wspeeds);
 			}
-			
-			close (fd);
 		}
 		
-	} /* !only_check_for_fs */
-
+		close (fd);
+	}
+		
 	ret = 0;
 
 	/* Also return 2 if we're a cdrom and we got a disc */
@@ -382,7 +429,7 @@ main (int argc, char *argv[])
 		close (fd);
         HAL_DEBUG (("PROBE CLOSED LOCK ON CDROM"));
 	} else {
-		struct volume_id *vid;
+		blkid_probe pr;
 		GDir *dir;
 		const gchar *partition;
 		const gchar *main_device;
@@ -452,25 +499,31 @@ main (int argc, char *argv[])
 		libhal_changeset_set_property_string (cs, "storage.partitioning_scheme", "none");
 
 		/* probe for file system */
-		vid = volume_id_open_fd (fd);
-		if (vid != NULL) {
-			if (volume_id_probe_all (vid, 0, size) == 0) {
+		pr = blkid_new_probe ();
+		if (pr != NULL) {
+			blkid_probe_set_request (pr, BLKID_PROBREQ_LABEL | BLKID_PROBREQ_UUID |
+						 BLKID_PROBREQ_TYPE | BLKID_PROBREQ_SECTYPE |
+						 BLKID_PROBREQ_USAGE | BLKID_PROBREQ_VERSION);
+			if (blkid_probe_set_device (pr, fd, 0, size) == 0 &&
+			    blkid_do_safeprobe (pr) == 0) {
+				const char *usage;
+
 				/* signal to hald that we've found something and a fakevolume
 				 * should be added - see hald/linux/blockdev.c:add_blockdev_probing_helper_done()
 				 * and hald/linux/blockdev.c:block_rescan_storage_done().
 				 */
-				if (vid->usage_id == VOLUME_ID_FILESYSTEM ||
-				    vid->usage_id == VOLUME_ID_RAID ||
-				    vid->usage_id == VOLUME_ID_OTHER ||
-				    vid->usage_id == VOLUME_ID_CRYPTO)
+				if (blkid_probe_lookup_value (pr, "USAGE", &usage, NULL) == 0 &&
+				    (strcmp (usage, "filesystem") == 0 ||
+				     strcmp (usage, "raid") == 0 ||
+				     strcmp (usage, "other") == 0 ||
+				     strcmp (usage, "crypto") == 0))
 					ret = 2;
-			} else {
-				;
 			}
-			volume_id_close(vid);
+			blkid_free_probe (pr);
 		}
 		close (fd);
 	}
+
 	
 out:
 	if (cs != NULL) {
