@@ -68,7 +68,7 @@ struct LibHalStoragePolicy_s {
 };
 
 LibHalStoragePolicy *
-libhal_storage_policy_new ()
+libhal_storage_policy_new (void)
 {
 	LibHalStoragePolicy *p;
 
@@ -191,15 +191,6 @@ out:
 	return result;
 }
 
-static void
-fixup_string (char *s)
-{
-	/* TODO: first strip leading and trailing whitespace */
-	/*g_strstrip (s);*/
-
-	/* TODO: could do nice things on all-upper case strings */
-}
-
 /* volume may be NULL (e.g. if drive supports removable media) */
 char *
 libhal_drive_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *volume, LibHalStoragePolicy *policy)
@@ -241,8 +232,6 @@ libhal_drive_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *volu
 		}
 	}
 
-	fixup_string (vendormodel_str);
-
 	if (drive_type==LIBHAL_DRIVE_TYPE_CDROM) {
 
 		/* Optical drive handling */
@@ -363,21 +352,13 @@ libhal_volume_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *vol
 	char *name;
 	char *size_str;
 	const char *volume_label;
-	const char *model;
-	const char *vendor;
 	LibHalDriveType drive_type;
-	dbus_bool_t drive_is_hotpluggable;
 	dbus_bool_t drive_is_removable;
-	LibHalDriveCdromCaps drive_cdrom_caps;
 	char buf[MAX_STRING_SZ];
 
 	volume_label = libhal_volume_get_label (volume);
-	model = libhal_drive_get_model (drive);
-	vendor = libhal_drive_get_vendor (drive);
 	drive_type = libhal_drive_get_type (drive);
-	drive_is_hotpluggable = libhal_drive_is_hotpluggable (drive);
 	drive_is_removable = libhal_drive_uses_removable_media (drive);
-	drive_cdrom_caps = libhal_drive_get_cdrom_caps (drive);
 
 	size_str = libhal_volume_policy_compute_size_as_string (volume);
 
@@ -431,7 +412,14 @@ libhal_volume_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *vol
 			else
 				name = strdup (_("DVD-R"));
 			break;
-			
+
+		case LIBHAL_VOLUME_DISC_TYPE_DVDR_DL:
+			if (libhal_volume_disc_is_blank (volume))
+				name = strdup (_("Blank DVD-R Dual-Layer"));
+			else
+				name = strdup (_("DVD-R Dual-Layer"));
+			break;
+
 		case LIBHAL_VOLUME_DISC_TYPE_DVDRW:
 			if (libhal_volume_disc_is_blank (volume))
 				name = strdup (_("Blank DVD-RW"));
@@ -512,6 +500,8 @@ libhal_volume_policy_compute_display_name (LibHalDrive *drive, LibHalVolume *vol
 			else
 				name = strdup (_("Magneto Optical"));
 		}
+		
+		goto out;
 	}
 
 	/* Fallback: size of media */
@@ -708,6 +698,7 @@ struct LibHalDrive_s {
 	dbus_bool_t is_hotpluggable;
 	dbus_bool_t is_removable;
 	dbus_bool_t is_media_detected;
+	dbus_bool_t is_media_detection_automatic;
 	dbus_bool_t requires_eject;
 
 	LibHalDriveType type;
@@ -778,7 +769,7 @@ struct LibHalVolume_s {
 	dbus_bool_t disc_is_rewritable;
 
 	unsigned int block_size;
-	unsigned int num_blocks;
+	dbus_uint64_t num_blocks;
 
 	char *desired_mount_point;
 	char *mount_filesystem;
@@ -978,6 +969,7 @@ libhal_drive_from_udi (LibHalContext *hal_ctx, const char *udi)
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.hotpluggable",      drive->is_hotpluggable);
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.removable",         drive->is_removable);
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.removable.media_available", drive->is_media_detected);
+		LIBHAL_PROP_EXTRACT_BOOL   ("storage.media_check_enabled", drive->is_media_detection_automatic);
 		LIBHAL_PROP_EXTRACT_UINT64 ("storage.removable.media_size", drive->drive_media_size); 
 		LIBHAL_PROP_EXTRACT_BOOL   ("storage.requires_eject",    drive->requires_eject);
 
@@ -1185,7 +1177,7 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 		LIBHAL_PROP_EXTRACT_STRING ("volume.crypto_luks.clear.backing_volume", vol->crypto_backing_volume);
 
 		LIBHAL_PROP_EXTRACT_INT    ("volume.block_size",         vol->block_size);
-		LIBHAL_PROP_EXTRACT_INT    ("volume.num_blocks",         vol->num_blocks);
+		LIBHAL_PROP_EXTRACT_UINT64 ("volume.num_blocks",         vol->num_blocks);
 		LIBHAL_PROP_EXTRACT_UINT64 ("volume.size", 		 vol->volume_size); 
 		LIBHAL_PROP_EXTRACT_STRING ("volume.label",              vol->volume_label);
 		LIBHAL_PROP_EXTRACT_STRING ("volume.mount_point",        vol->mount_point);
@@ -1229,6 +1221,8 @@ libhal_volume_from_udi (LibHalContext *hal_ctx, const char *udi)
 			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_DVDR;
 		} else if (strcmp (disc_type_textual, "dvd_rw") == 0) {
 			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_DVDRW;
+		} else if (strcmp (disc_type_textual, "dvd_r_dl") == 0) {
+			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_DVDR_DL;
 		} else if (strcmp (disc_type_textual, "dvd_plus_r") == 0) {
 			vol->disc_type = LIBHAL_VOLUME_DISC_TYPE_DVDPLUSR;
 		} else if (strcmp (disc_type_textual, "dvd_plus_rw") == 0) {
@@ -1510,7 +1504,7 @@ libhal_volume_get_size (LibHalVolume *volume)
 	if (volume->volume_size > 0)
 		return volume->volume_size;
 	else
-		return ((dbus_uint64_t)volume->block_size) * ((dbus_uint64_t)volume->num_blocks);
+		return ((dbus_uint64_t)volume->block_size) * volume->num_blocks;
 }
 
 dbus_uint64_t
@@ -1538,6 +1532,12 @@ libhal_drive_is_media_detected (LibHalDrive *drive)
 	return drive->is_media_detected;
 }
 
+dbus_bool_t
+libhal_drive_is_media_detection_automatic (LibHalDrive *drive)
+{
+	return drive->is_media_detection_automatic;
+}
+
 dbus_uint64_t
 libhal_drive_get_size (LibHalDrive *drive)
 {
@@ -1958,7 +1958,7 @@ libhal_drive_policy_default_get_managed_keyword_secondary (LibHalContext *hal_ct
 dbus_bool_t
 libhal_drive_policy_is_mountable (LibHalDrive *drive, LibHalStoragePolicy *policy)
 {
-	printf ("should_mount=%d, no_partitions_hint=%d\n", drive->should_mount, drive->no_partitions_hint);
+	printf ("should_mount=%u, no_partitions_hint=%u\n", drive->should_mount, drive->no_partitions_hint);
 
 	return drive->should_mount && drive->no_partitions_hint;
 }
@@ -1984,7 +1984,7 @@ mopts_collect (LibHalContext *hal_ctx, const char *namespace, int namespace_len,
 	LibHalPropertySetIterator it;
 	DBusError error;
 
-	if(hal_ctx == 0) {
+	if(hal_ctx == NULL) {
 		fprintf (stderr,"%s %d : LibHalContext *ctx is NULL\n",__FILE__, __LINE__);
 		return;
 	}
@@ -1999,10 +1999,8 @@ mopts_collect (LibHalContext *hal_ctx, const char *namespace, int namespace_len,
 	}
 
 	for (libhal_psi_init (&it, properties); libhal_psi_has_more (&it); libhal_psi_next (&it)) {
-		int type;
 		char *key;
 		
-		type = libhal_psi_get_type (&it);
 		key = libhal_psi_get_key (&it);
 		if (libhal_psi_get_type (&it) == LIBHAL_PROPERTY_TYPE_BOOLEAN &&
 		    strncmp (key, namespace, namespace_len - 1) == 0) {
